%---------------------------------------------------%
% Foot Motion Problem:                              %
%---------------------------------------------------%

clear all; close all;

handLoadControls = true;
springPosParams = false;
optimizeSynergyWeights = false;
% addpath('C:\Files\GPOPS-II New\nlp\snopt')

% Load model parameters determined by past calibration runs
cdorig = cd;
cd('../SetupFiles')
auxdata = SetupOpenSimOptimalTracking(0,true);
cd(cdorig)

auxdata.handLoadControls = handLoadControls;
auxdata.springPosParams = springPosParams;
auxdata.optimizeSynergyWeights = optimizeSynergyWeights;

% Load initial and final model pose and time
t0 = auxdata.time(1);
tf = auxdata.time(end);
time = auxdata.time;
q0 = auxdata.q(1,:);
qp0 = auxdata.qp(1,:);
qf = auxdata.q(end,:);
qpf = auxdata.qp(end,:);

% Set initial guesses for Loads (control) and joint angle trajectories
% (states)
loadsGuess = auxdata.idData;
qGuess = auxdata.q;
qpGuess = auxdata.qp;
qppGuess = auxdata.qpp;
qpppGuess = auxdata.qppp;
nJerkControls = size(qpppGuess,2);
auxdata.nJerkControls = nJerkControls;

% Determine the range of the states
qRange = max(qGuess) - min(qGuess);
qpRange = max(qpGuess) - min(qpGuess);
qppRange = max(qppGuess) - min(qppGuess);
qpppRange = max(qpppGuess) - min(qpppGuess);

auxdata.activeDofs = logical(1-(qRange == 0));
auxdata.activeDofs(1:6) = [];

% Give locked DOFs a range of motion
qpppRange(qRange == 0) = 40000*pi/180;
qppRange(qRange == 0) = 4000*pi/180;
qpRange(qRange == 0) = 400*pi/180;
qRange(qRange == 0) = 60*pi/180;

% Set bounds on the states based on the previously calcualted range
qMin = min(qGuess) - 2*qRange;  
qMax  = max(qGuess) + 2*qRange;
qpMin = min(qpGuess) - 1.5*qpRange; 
qpMax  = max(qpGuess) + 1.5*qpRange;
qppMin = min(qppGuess) - 1*qppRange; 
qppMax  = max(qppGuess) + 1*qppRange;
qpppMin = min(qpppGuess) - 1*qpppRange; % 0.5 %NOTE (Nathan): This is different than previous step (ground contact calibration)
qpppMax  = max(qpppGuess) + 1*qpppRange; %0.5

% Determing the range of the controls
loadsRange = max(auxdata.idData) - min(auxdata.idData);
torquesRange = max(auxdata.torques) - min(auxdata.torques);

% Give bigger range to DOFs with small loads (for instance the toes)
loadsRange(loadsRange<1/(auxdata.tScale^2)) = 25/(auxdata.tScale^2);
torquesRange(torquesRange<1/(auxdata.tScale^2)) = 25/(auxdata.tScale^2);

% Set bounds on the torques (for when using extra controls)
loadsMin = min(auxdata.idData)-2*loadsRange; 
loadsMax = max(auxdata.idData)+2*loadsRange;

torquesMin = min(auxdata.torques) - 2*torquesRange;
torquesMax = max(auxdata.torques) + 2*torquesRange;

torquesGuess = auxdata.torques;

% Set weights for marker errors (the model tracks markers from experimental
% data
% MkrNamesOSIM = {'R.Shoulder' 'L.Shoulder' 'Neck' 'Chest' 'R.Elbow' 'R.Wrist' 'L.Elbow' 'L.Wrist' 'R.ASIS' 'L.ASIS'...
%     'Sacral' 'R.Thigh.Superior' 'R.Thigh.Inferior' 'R.Thigh.Lateral' 'R.Shank.Superior' 'R.Shank.Inferior' 'R.Shank.Lateral'...
%     'R.Heel' 'R.Midfoot.Superior' 'R.Midfoot.Lateral' 'R.Toe' 'L.Thigh.Superior' 'L.Thigh.Inferior' 'L.Thigh.Lateral'...
%     'L.Shank.Superior' 'L.Shank.Lateral' 'L.Shank.Inferior' 'L.Heel' 'L.Midfoot.Superior' 'L.Midfoot.Lateral' 'L.Toe'...
%     'L.PSIS' 'R.PSIS'};
auxdata.errorWeights = ones(1,numel(auxdata.mkrNames));
auxdata.errorWeights([18 21 28 31]) = 10;
auxdata.errorWeights([3:8]) = .1;

auxdata.errorWeights = repmat(auxdata.errorWeights,3,1);
auxdata.errorWeights = auxdata.errorWeights(:)';

% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt3_GPOPS.osim');
opensimID_OMP('Subject1_v4pt3_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt3_GPOPS.osim');

% Get median hand positions
% handForceLoc = [0.01,-0.07,0; 0.01,-0.07,0]; % NOTE (Nathan): Make sure that this is the same location as where the force is applied in the model
handForceLoc = [0.01,-0.06,0; 0.01,-0.06,0]; % NOTE (Nathan): Make sure that this is the same location as where the force is applied in the model

% markerInRadiusPos = [-0.0358 -0.2376 0.0171; -0.0351 -0.2512 -0.0149];
% auxdata.markerInRadiusPos = markerInRadiusPos;
% avgMarkerPos = [0.7913 1.1486 0.6033; 0.8061 1.1710 0.1651];
% auxdata.avgMarkerPos = avgMarkerPos;
% [radiusPos, radiusVel, radiusAng, radiusAngVel] = opensimGetBodyPosVel_OMP(time,[qGuess qpGuess],markerInRadiusPos,auxdata.radiusBodyMat);

auxdata.handForceLoc = handForceLoc;
handBodyMat = auxdata.handBodyMat;
% % [handPos, handVel, handAng, handAngVel] = opensimGetBodyPosVel_OMP(time,[qGuess qpGuess],handBodyMat);
[handPos, handVel, handAng, handAngVel] = opensimGetBodyPosVel_OMP(time,[qGuess qpGuess],handForceLoc,handBodyMat);
% 
% medianHandPos = median(handPos);
% medianHandAng = median(handAng);
% auxdata.medianHandPos_r = medianHandPos(1:3);
% auxdata.medianHandPos_l = medianHandPos(4:6);
% auxdata.medianHandAng_r = medianHandAng(1:3);
% auxdata.medianHandAng_l = medianHandAng(4:6);

% auxdata.posOnHandR = [0.01,-0.07,0; 0.01,-0.07,0];
auxdata.posOnBarR = [0.874274080385552,1.12880647759450,0.602273837458135]; %NOTE (Nathan): These come from the getWristMarkerLocs program in SetupFiles
auxdata.posOnBarL = [0.869472149835813,1.13141862779562,0.164197648932865];

% Min and Max hand loads allowed
% NOTE (Nathan): Thses are only used if the hand loads are minimized or for
% hand load controls
handLoadMin = -[200*ones(1,3),50*ones(1,3),200*ones(1,3),50*ones(1,3)];
handLoadMax = [200*ones(1,3),50*ones(1,3),200*ones(1,3),50*ones(1,3)];
handLoadRange = handLoadMax - handLoadMin;


% % Marker Error
% % NOTE (Nathan): I don't think these do anything
% mkrErrorsAllowed = 2e-2*ones(1,numel(auxdata.mkrNames)); %Andrew had 1e-2
% mkrErrorsAllowed([18:21 28:31]) = 1.5e-2; %Andrew had 2.5e-3
% mkrErrorsAllowed([3:8]) = 3e-2; %Andrew had 1.5e-2
% mkrErrorsAllowed = repmat(mkrErrorsAllowed, 3,1);% Duplicate for each dimension
% mkrErrorsAllowed = mkrErrorsAllowed(:)';

% GRF Errors
grfErrorsInt = 1*[1e1 2e1 1e1 1e1 2e1 1e1 10*ones(1,3) 10*ones(1,3)]; %5e-1* ... 5 %NOTE (Nathan): The torques are different than previous steps %Used for setting path constraint on max error of a GRF
grfErrorsMax = 3*grfErrorsInt; % 2 works better than 3 some times %Used for setting path constraint on max error of a GRF

% Coordinate Errors
qErrorsInt = 3*(pi/180)*ones(1,31);
qErrorsInt(1:3) = 1*qRange(1:3); %1*
qErrorsInt(4:6) = 1; %100
qErrorsInt([9 13 16 20:31]) = 0.25*qRange([9 13 16 20:31]); %0.25*
% qErrorsInt([9 13 16 20]) = 0.25*qRange([9 13 16 20]); %0.25*
qErrorsInt(24:31) = 200*qRange(24:31);


qErrorsMax = 5*qErrorsInt; %5
qErrorsMax([13,20]) = (10*pi/180)*ones(1,2);

% Belt Speed and Number of Springs
auxdata.beltSpeed = 0.6;
auxdata.numSprings = sum(auxdata.numSpringsBody);


% Setup scale factors
auxdata.tMin = t0;
auxdata.tMax = tf;
auxdata.stateMin = [qMin,qpMin,qppMin];
auxdata.stateMax = [qMax,qpMax,qppMax];
if handLoadControls
    auxdata.controlMin = [qpppMin,zeros(1,10),handLoadMin];
    auxdata.controlMax = [qpppMax,1*ones(1,10),handLoadMax];
    auxdata.integralMin = 0*ones(1,31+6+6+12+72+10+12+31);
    auxdata.integralMax = auxdata.tMax*([qErrorsInt ...
        grfErrorsInt...
        1*loadsRange([7:10,12:13,15:18,20:21])... % [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
        0.5*(0.25+max(auxdata.acts))...
        3*ones(1,10)... % For minimizing synergy commands (in case of indeterminicy or something)
        handLoadRange...
        1*qpppRange]).^2; % NOTE (Nathan): This was 1* in previous step % Ignore locked coordinates (wrists) and patella OLD: all
    
else
    auxdata.controlMin = [qpppMin,zeros(1,10)];
    auxdata.controlMax = [qpppMax,1*ones(1,10)];
    auxdata.integralMin = 0*ones(1,31+6+6+12+72+12+31);
    auxdata.integralMax = auxdata.tMax*([qErrorsInt ...
        grfErrorsInt...
        1*loadsRange([7:10,12:13,15:18,20:21])... % [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
        0.5*(0.25+max(auxdata.acts))...
        handLoadRange ...
        1*qpppRange]).^2; % NOTE (Nathan): This was 1* in previous step % Ignore locked coordinates (wrists) and patella OLD: all
end

onescol = ones(length(auxdata.time),1);
numTorques = size(auxdata.torques,2);
pathMult = 1e0;
actsMin = zeros(1,2*36);
actsMax = ones(1,2*36);
auxdata.pathMin = 1*[pathMult*[-1e-2*ones(1,3) -1e0*ones(1,3) -1e-1*ones(1,10)] -qErrorsMax -grfErrorsMax -0.005*ones(1,6)];
auxdata.pathMax = 1*[pathMult*[1e-2*ones(1,3) 1e0*ones(1,3) 1e-1*ones(1,10)] qErrorsMax grfErrorsMax 0.005*ones(1,6)];
% auxdata.pathMin = [pathMult*[-1e-2*ones(1,3) -1e0*ones(1,3) -1e-1*ones(1,10)] -qErrorsMax -grfErrorsMax actsMin];
% auxdata.pathMax = [pathMult*[1e-2*ones(1,3) 1e0*ones(1,3) 1e-1*ones(1,10)] qErrorsMax grfErrorsMax actsMax];
% auxdata.pathMin = pathMult*[-1e-2*ones(1,3) -1e0*ones(1,3)];
% auxdata.pathMax = pathMult*[1e-2*ones(1,3) 1e0*ones(1,3)];

% if optimizeSynergyWeights
%     auxdata.parammin = zeros(1,2*5*36); % 2: Two legs, 5: Five synergies, 36: Thirty-six muscles
%     auxdata.parammax = 2*ones(1,2*5*36);
% else
%     load synergiesOpt.mat;
%     auxdata.synergyWeights_r = synergyWeights_r;
%     auxdata.synergyWeights_l = synergyWeights_l;
% end

if ~optimizeSynergyWeights    
    load synergiesOpt.mat
    auxdata.synergyWeights_r = synergyWeights_r;
    auxdata.synergyWeights_l = synergyWeights_l;
    eventMin = [];
    eventMax = [];
else
    auxdata.parammin = zeros(1,2*5*36); % 2: Two legs, 5: Five synergies, 36: Thirty-six muscles
    auxdata.parammax = 4*ones(1,2*5*36);
    eventMin = 3.96*ones(1,10);
    eventMax = 4.04*ones(1,10);
end

if handLoadControls
    auxdata.eventMin = [[zeros(1,25)-0.5*pi/180 zeros(1,25)-1*pi/180 -2*[10 25 10 10 25 10] pathMult*[-1e-1*ones(1,3) -1e0*ones(1,3)]] eventMin]; % NOTE (Nathan): The 2* wasn't there last step
    auxdata.eventMax = [[zeros(1,25)+0.5*pi/180 zeros(1,25)+1*pi/180 2*[10 25 10 10 25 10] pathMult*[1e-1*ones(1,3) 1e0*ones(1,3)]] eventMax]; % NOTE (Nathan): The 2* wasn't there last step
else
    auxdata.eventMin = [zeros(1,25)-0.5*pi/180 zeros(1,25)-1*pi/180 -2*[10 25 10 10 25 10], -2*[10 25 10 10 25 10] pathMult*[-1e-1*ones(1,3) -1e0*ones(1,3)]]; % NOTE (Nathan): The 2* wasn't there last step
    auxdata.eventMax = [zeros(1,25)+0.5*pi/180 zeros(1,25)+1*pi/180 2*[10 25 10 10 25 10], 2*[10 25 10 10 25 10] pathMult*[1e-1*ones(1,3) 1e0*ones(1,3)]]; % NOTE (Nathan): The 2* wasn't there last step
end


% Load Previous Solution
load('solution_synergyCalibration_snopt_c1v4pt5.mat');

if 0 %Run Opt    
%-------------------------------------------------------------------------%
%----------------------- Setup for Problem Bounds ------------------------%
%-------------------------------------------------------------------------%

nStates = length(auxdata.stateMin);
nControls = length(auxdata.controlMin);
% nparams = length(auxdata.paramsmin);

iphase = 1;
bounds.phase(iphase).initialtime.lower = -.5;
bounds.phase(iphase).initialtime.upper = -.5;
bounds.phase(iphase).finaltime.lower = .5;
bounds.phase(iphase).finaltime.upper = .5;
% states are the joint angles, velocities, accelerations and the joint
% torques
bounds.phase(iphase).initialstate.lower = -.5*ones(1,nStates);%,vmin];
bounds.phase(iphase).initialstate.upper = .5*ones(1,nStates);%,vmax];
bounds.phase(iphase).finalstate.lower = -.5*ones(1,nStates);%,vmin];%[qf,qpf];
bounds.phase(iphase).finalstate.upper = .5*ones(1,nStates);%,vmax];%[qf,qpf];
bounds.phase(iphase).state.lower = -.5*ones(1,nStates);%,vmin];
bounds.phase(iphase).state.upper = .5*ones(1,nStates);%,vmax];
% Dynamics equations are forced to be satisfied by path constraints
% Control with joint angle jerk and the time derivative of the joint loads
bounds.phase(iphase).control.lower = -.5*ones(1,nControls);
bounds.phase(iphase).control.upper = .5*ones(1,nControls);
% bounds.phase(iphase).control.lower = [-0.5*ones(1,length(qpppMin)),-0.5*ones(1,10),-0.5*ones(1,length(handLoadMin))];
% bounds.phase(iphase).control.upper = [0.5*ones(1,length(qpppMax)),0.5*ones(1,10),0.5*ones(1,length(handLoadMax))];
bounds.phase(iphase).path.lower = -0.5*ones(size(auxdata.pathMin)); % Bounds on pelvis residuals
bounds.phase(iphase).path.upper = 0.5*ones(size(auxdata.pathMax));
bounds.phase(iphase).integral.lower = 0*(ones(size(auxdata.integralMin)));
bounds.phase(iphase).integral.upper = 1*(ones(size(auxdata.integralMax)));
if optimizeSynergyWeights
    bounds.parameter.lower = -0.5*ones(1,2*5*36);
    bounds.parameter.upper = 0.5*ones(1,2*5*36);
end

%-------------------------------------------------------------------------%
%---------------------- Provide Guess of Solution ------------------------%
%-------------------------------------------------------------------------%
% solution.phase.control(:,30:99) = solution.phase.control(:,30:99)/10;
% Set initial guess either from previous optimal control solution or from
% the auxilary data which was calculated from a different model
iphase = 1;
if 1
    guess = solution;
%     if optimizeSynergyWeights
%         guess.parameter = [auxdata.synergyWeights_r(1,:),auxdata.synergyWeights_r(2,:),auxdata.synergyWeights_r(3,:),...
%             auxdata.synergyWeights_r(4,:),auxdata.synergyWeights_r(5,:),auxdata.synergyWeights_l(1,:),...
%             auxdata.synergyWeights_l(2,:),auxdata.synergyWeights_l(3,:),auxdata.synergyWeights_l(4,:),auxdata.synergyWeights_l(5,:)]; % Must be active for model calibration
%     end
else
    guess.phase(iphase).time    = ([auxdata.time]-(auxdata.tMax+auxdata.tMin)/2)./(auxdata.tMax-auxdata.tMin);
    guess.phase(iphase).state   = ([qGuess qpGuess qppGuess]-onescol*(auxdata.stateMax+auxdata.stateMin)/2)./(onescol*(auxdata.stateMax-auxdata.stateMin));% auxdata.Loads];% auxdata.Loadsp]
    if handLoadControls
        guess.phase(iphase).control = ([qpppGuess,auxdata.synergyCommands_r,auxdata.synergyCommands_l,onescol*zeros(1,12)]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
    else
        guess.phase(iphase).control = ([qpppGuess,auxdata.synergyCommands_r,auxdata.synergyCommands_l]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
    end
    if optimizeSynergyWeights
        guess.parameter = [auxdata.synergyWeights_r(1,:),auxdata.synergyWeights_r(2,:),auxdata.synergyWeights_r(3,:),...
            auxdata.synergyWeights_r(4,:),auxdata.synergyWeights_r(5,:),auxdata.synergyWeights_l(1,:),...
            auxdata.synergyWeights_l(2,:),auxdata.synergyWeights_l(3,:),auxdata.synergyWeights_l(4,:),auxdata.synergyWeights_l(5,:)]; % Must be active for model calibration
    end
    guess.phase(iphase).integral = (10 - (auxdata.integralMax+auxdata.integralMin)/2)./(auxdata.integralMax - auxdata.integralMin);
    
end


%-------------------------------------------------------------------------%
%------------- Set up Event Constraints That Link Phases -----------------%
%-------------------------------------------------------------------------%
bounds.eventgroup(1).lower = -0.5*ones(size(auxdata.eventMin));
bounds.eventgroup(1).upper = 0.5*ones(size(auxdata.eventMax));


%-------------------------------------------------------------------------%
%------------- Assemble Information into Problem Structure ---------------%
%-------------------------------------------------------------------------%
setup.name = 'openSim Full Body Motion';
setup.functions.continuous = @osimOptimalTrackingContinuous;
auxdata.ContinuousFunc = setup.functions.continuous;
setup.functions.endpoint = @osimOptimalTrackingEndpoint;
setup.auxdata = auxdata;
setup.bounds = bounds;
setup.guess = guess;
setup.nlp.solver = 'snopt';
% setup.nlp.ipoptoptions.linear_solver = 'ma57';
setup.nlp.ipoptoptions.tolerance = 1e-4;
setup.nlp.snoptoptions.tolerance = 1e-5;
setup.nlp.ipoptoptions.maxiterations = 500;
% setup.derivatives.stepsize1 = 1e-9;
setup.derivatives.supplier = 'sparseFD';
setup.derivatives.derivativelevel = 'first';
setup.derivatives.dependencies = 'sparse';
% setup.scales.method='automatic-bounds';
mesh.method = 'hp-PattersonRao';
mesh.tolerance       = 1e-4;
mesh.maxiterations    = 0;
mesh.colpointsmin    = 2;
mesh.colpointsmax    = 10;
N                    = 10; %10
mesh.phase.colpoints = 5*ones(1,N); %5
mesh.phase.fraction  = ones(1,N)/N;
setup.method = 'RPM-integration';
setup.mesh = mesh;

%-------------------------------------------------------------------------%
%------------------------- Solve Problem Using GPOP2 ---------------------%
%-------------------------------------------------------------------------%
tic
output = gpops2(setup);
toc
solution = output.result.solution;
save solution_synergyCalibration_snopt_c1v4pt5.mat solution
save output_synergyCalibration_snopt_c1v4pt5.mat output

end

%-------------------------------------------------------------------------%
%------------------------------- Plot Solution ---------------------------%
%-------------------------------------------------------------------------%

%load output_osimOptTracking_calibrateSprings4.mat

tScale = 1;%auxdata.tScale;

input = solution;
if optimizeSynergyWeights
    input.phase.parameter = solution.parameter;
end
input.auxdata = auxdata;

% outputContinuous = FullBodyOptimalControlContinuous_experimentalGRFs_Markers(input);
outputContinuous = osimOptimalTrackingContinuous(input);

onescol = ones(size(solution.phase.state(:,1)));

solution.phase.time = (solution.phase.time)*(auxdata.tMax-auxdata.tMin)+(auxdata.tMax+auxdata.tMin)/2;
solution.phase.state = (solution.phase.state).*(onescol*(auxdata.stateMax-auxdata.stateMin))+onescol*(auxdata.stateMax+auxdata.stateMin)/2;
solution.phase.control = (solution.phase.control).*(onescol*(auxdata.controlMax-auxdata.controlMin))+onescol*(auxdata.controlMax+auxdata.controlMin)/2;
if optimizeSynergyWeights
    solution.parameter = (solution.parameter).*((auxdata.parammax-auxdata.parammin))+(auxdata.parammax+auxdata.parammin)/2;
end

mkrDataSplined = interp1(auxdata.time,auxdata.mkrData,solution.phase.time,'pchip');

mkrDataSplined = fnval(auxdata.spMkrData,solution.phase.time)';

error = mkrDataSplined-outputContinuous.markers;

% Calculate RMS error for each marker at each time frame
for i = 1:size(mkrDataSplined,2)/3
    RMSError(:,i) = sum(error(:,(i-1)*3+1:3*i).^2,2).^(1/2);
end

GRFsSplit = outputContinuous.GRFsSplit;
figure % 1
plot(solution.phase.time/solution.phase.time(end),GRFsSplit(:,1:3)+GRFsSplit(:,4:6),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataR(:,1:3),'b')
title('Right GRFs');

figure % 2
plot(solution.phase.time/solution.phase.time(end),GRFsSplit(:,13:15)+GRFsSplit(:,16:18),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataL(:,1:3),'b')
title('Left GRFs');

figure % 3
plot(solution.phase.time/solution.phase.time(end),GRFsSplit(:,7:9)+GRFsSplit(:,10:12),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataR(:,4:6),'b')
title('Right GRMs');

figure % 4
plot(solution.phase.time/solution.phase.time(end),GRFsSplit(:,19:21)+GRFsSplit(:,22:24),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataL(:,4:6),'b')
title('Left GRMs');

figure % 5
for i = 1:29
    if ismember(i,4:6)
        midpoint = ((max(qGuess(:,i)) - min(qGuess(:,i)))/2 + min(qGuess(:,i)));
        minpoint = midpoint - 0.1;
        maxpoint = midpoint + 0.1;
        subplot(5,6,i)
        plot(solution.phase.time/solution.phase.time(end), solution.phase.state(:,i),'r')
        hold on
    %     plot(auxdata.time/auxdata.time(end), auxdata.q(:,i),'b')
        plot(auxdata.time/auxdata.time(end), qGuess(:,i),'b')
        ylim([minpoint,maxpoint]);
        title(strrep(auxdata.qLabels(i),'_',' '));
    else
        midpoint = (180/pi)*((max(qGuess(:,i)) - min(qGuess(:,i)))/2 + min(qGuess(:,i)));
        minpoint = midpoint - 0.75*180/pi;
        maxpoint = midpoint + 0.75*180/pi;
        subplot(5,6,i)
        plot(solution.phase.time/solution.phase.time(end), solution.phase.state(:,i)*180/pi,'r')
        hold on
    %     plot(auxdata.time/auxdata.time(end), auxdata.q(:,i),'b')
        plot(auxdata.time/auxdata.time(end), qGuess(:,i)*180/pi,'b')
        ylim([minpoint,maxpoint]);
        title(strrep(auxdata.qLabels(i),'_',' '));
    end
end

figure % 6
for i = 1:29
    
    subplot(5,6,i)
    plot(solution.phase.time/solution.phase.time(end), solution.phase.control(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.qpp(:,i),'b')
    plot(auxdata.time/auxdata.time(end), qpppGuess(:,i),'b')
    title(strrep(auxdata.qLabels(i),'_',' '));
end

figure % 7
for i = 1:23
    subplot(5,5,i)
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.torques(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.idData(:,i),'b')
    plot(auxdata.time/auxdata.time(end), auxdata.torques(:,i),'b')
    title(strrep(auxdata.torqueNames(i),'_',' '));
end

figure % 8
for iii = 1:3
    subplot(2,3,iii);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsR(:,iii));
    subplot(2,3,iii+3);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsL(:,iii));
    title('Hand Forces');
end

figure % 9
for iii = 1:3
    subplot(2,3,iii);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsR(:,iii+3));
    subplot(2,3,iii+3);
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.handLoadsL(:,iii+3));
    title('Hand Torques');
end


neuralCommands = solution.phase.control(:,32:41);
neuralCommands_r = neuralCommands(:,1:5);
neuralCommands_l = neuralCommands(:,6:10);
if optimizeSynergyWeights
    synergyWeights_r = zeros(5,36);
    synergyWeights_l = zeros(5,36);
    for iii = 1:5
        synergyWeights_r(iii,:) = solution.parameter(((iii-1)*36 + 1):(iii*36));
        synergyWeights_l(iii,:) = solution.parameter(((iii-1)*36 + 181):(iii*36 + 180));
    end
else
    synergyWeights_r = auxdata.synergyWeights_r;
    synergyWeights_l = auxdata.synergyWeights_l; %reshape(input.phase.parameter(1,:),10,35);
end
synergyTime = solution.phase.time;
save synergiesOpt.mat neuralCommands_r neuralCommands_l synergyWeights_r synergyWeights_l synergyTime


activations(:,1:36) = neuralCommands(:,1:5)*synergyWeights_r;
activations(:,37:72) = neuralCommands(:,6:10)*synergyWeights_l;

actsExp = fnval(auxdata.spActs,auxdata.time(end)*solution.phase.time/solution.phase.time(end))';
vaf = zeros(1,72);
for iii = 1:72
    vaf(iii) = 1 - var(actsExp(:,iii) - activations(:,iii))/var(actsExp(:,iii));
end

figure %10
title('Activation Matching')
for i = 1:36
    subplot(6,6,i)
    plot(solution.phase.time/solution.phase.time(end), activations(:,i),'r')
    hold on
    plot(auxdata.time/auxdata.time(end), auxdata.acts(1:101,i),'b')
    axis([0 1 0 1])
    title(strrep(auxdata.muscleNames(i),'_',' '));
end

figure %11
title('Activation Matching')
for i = 1:36
    subplot(6,6,i)
    plot(solution.phase.time/solution.phase.time(end), activations(:,i+36),'r')
    hold on
    plot(auxdata.time/auxdata.time(end), auxdata.acts(1:101,i+36),'b')
    axis([0 1 0 1])
    title(strrep(auxdata.muscleNames(i+36),'_',' '));
end

figure %12
title('Synergy Commands (R)')
for iii = 1:5
    subplot(2,3,iii)
    plot(solution.phase.time/solution.phase.time(end),neuralCommands(:,iii));
    axis([0 1 0 1])
end

figure %13
title('Synergy Commands (L)')
for iii = 1:5
    subplot(2,3,iii)
    plot(solution.phase.time/solution.phase.time(end),neuralCommands(:,iii+5));
    axis([0 1 0 1])
end

%% Output Kinematics for OpenSim
outputIKMot('SynergyCalibration_Trial04_Cycle12.mot',solution.phase.state(:,1:31),solution.phase.time);

[spTorques] = spaps(solution.phase.time,outputContinuous.torques',0.01);
torques = fnval(spTorques,auxdata.time)';
[spq] = spaps(solution.phase.time,solution.phase.state(:,1:31)',0.0001);
q = fnval(spq,auxdata.time)';
[spqp] = spaps(solution.phase.time,solution.phase.state(:,(1:31)+31)',0.001);
qp = fnval(spqp,auxdata.time)';
[spqpp] = spaps(solution.phase.time,solution.phase.state(:,(1:31)+62)',0.1);
qpp = fnval(spqpp,auxdata.time)';

spHandLoadsR = spaps(solution.phase.time,outputContinuous.handLoadsR',0.001);
spHandLoadsL = spaps(solution.phase.time,outputContinuous.handLoadsL',0.001);
handLoadsR = fnval(spHandLoadsR,auxdata.time)';
handLoadsL = fnval(spHandLoadsL,auxdata.time)';

% [spq, qNewData] = spaps(auxdata.time,q',0.0000001);
% [spqp, qpNewData] = spaps(auxdata.time,qp',0.001);
% [spqpp, qppNewData] = spaps(auxdata.time,qpp',0.1);
% 
% q = fnval(spq,solution.phase.time)';
% qp = fnval(spqp,solution.phase.time)';
% qpp = fnval(spqpp,solution.phase.time)';
% 
% figure;
% for iii = 1:31
%     subplot(6,6,iii);
%     plot(solution.phase.time,q(:,iii),'b');
%     hold on
%     plot(solution.phase.time,solution.phase.state(:,iii),'r')
% end
% 
% figure;
% for iii = 1:31
%     subplot(6,6,iii);
%     plot(solution.phase.time,qp(:,iii),'b');
%     hold on
%     plot(solution.phase.time,solution.phase.state(:,iii+31),'r')
% end
% 
% figure;
% for iii = 1:31
%     subplot(6,6,iii);
%     plot(solution.phase.time,qpp(:,iii),'b');
%     hold on
%     plot(solution.phase.time,solution.phase.state(:,iii+62),'r')
% end
% 
% figure;
% for iii = 1:6
%     subplot(2,3,iii);
%     plot(solution.phase.time,outputContinuous.handLoadsR(:,iii),'r');
%     hold on
%     plot(auxdata.time,handLoadsR(:,iii),'b');
% end

save GPOPSResults_Trial04_Cycle12.mat torques q qp qpp
save handLoads_Trial04_Cycle12.mat handLoadsR handLoadsL spHandLoadsR spHandLoadsL

[spIDnew] = spaps(solution.phase.time,outputContinuous.torques',.5);

control = fnval(spIDnew,auxdata.time)';

NewID = control(:,[8 9 11 12 13 1 2 4 5 6]);

save Control_Trial04_Cycle12.mat control
save NewIDLoads_Trial04_Cycle12.mat NewID
